PROGRAM     BlckSize;

USES        Objects,
            PxEngine,
            OOPXENG,
            PXOOPMSG,
            OpString,
            OpDos;

VAR         dDatabase  : TDatabase;
            dEngine    : TEngine;

FUNCTION    PxErrMsg(LastError : INTEGER ; S : STRING) : WORD;
BEGIN
 PxErrMsg := 0;
 IF LastError <> 0 THEN
  WRITELN('Error code ' + Long2Str(LastError) + ':  ' +
	  PxOopErrMsg(LastError) + ' (' + S + ')')
END;

FUNCTION    PxErrMsgHalt(LastError : RetCode ; Msg : STRING) : WORD;
BEGIN
 PxErrMsgHalt := PxErrMsg(LastError,Msg);
 IF LastError <> PXSUCCESS THEN
  BEGIN
 { WRITELN('Fatal Error, shutting down'); }
   HALT(LastError)
  END
END;

PROCEDURE   InitPxEngine(BlockSize : INTEGER);
VAR	    PxEnv : TEnv;
BEGIN
 FILLCHAR(PxEnv,SIZEOF(PxEnv),0);
 PxEnv.EngineType := PxLocal;
 {$IFDEF DPMI}
 PxEnv.BufSize    := 256; { 8 - 256  32}
 PxEnv.MaxTables  :=  64; { 1 -  64   5}
 PxEnv.MaxRecBufs := 512; { 1 - 512  10}
 PxEnv.MaxFiles   := 255; { 3 - 255  10}
 {$ELSE}
 PxEnv.BufSize    :=  64; { 8 - 256  32}
 PxEnv.MaxTables  :=  32; { 1 -  64   5}
 PxEnv.MaxRecBufs :=  32; { 1 - 512  10}
 PxEnv.MaxFiles   :=  32; { 3 - 255  10}
 {$ENDIF}
 dEngine.Init(@PxEnv);
 PxErrMsgHalt(dEngine.LastError,'Engine.Init');
 dEngine.SetMaxTblSize(BlockSize * 64);
 PxErrMsgHalt(dEngine.LastError,'MaxTblSize');
{dEngine.SetTblCreateMode(px35fmt);}
 dDatabase.Init(@dEngine);
 PxErrMsgHalt(dDatabase.LastError,'Database.Init')
END;

PROCEDURE   DonePxEngine;
BEGIN
 dDatabase.Done;
 dEngine.Done;
END;

FUNCTION    RemoveExt(S : STRING) : STRING;
VAR         P : BYTE;
BEGIN
 P := POS('.',S);
 IF P <> 0 THEN RemoveExt := COPY(S,1,P-1)
 ELSE RemoveExt := S
END;

PROCEDURE   ConvertSecondaryIndexes(StructTblName,TargetTblName : STRING);
VAR         ExtNo     : INTEGER;
            FldName   : STRING;
            CaseSen   : BOOLEAN;
            IndexId   : FieldNumber;
BEGIN
 FOR ExtNo := 1 TO dDatabase.GetFieldCount(StructTblName) DO
  IF dDatabase.GetSKeyInfo(StructTblName + '.x' + HexB(ExtNo),
                                 FldName,
                                 CaseSen,
                                 IndexId) = PXSUCCESS THEN
   BEGIN
    WRITELN('Converting ' + StructTblName + '.x' + HexB(ExtNo),'  ',
            FldName,CaseSen:8,IndexId:6);
    PxErrMsg(dDatabase.CreateSIndex(TargetTblName,
                                    IndexId,
                                    pxIncSecondary),'CreateSIndex')
   END
  ELSE
   IF dDatabase.LastError <> PXERR_FILENOTFOUND THEN
    pxErrMsg(dDatabase.LastError,'GetSKeyInfo')
END;

PROCEDURE   ConvertCompositeIndexes(StructTblName,TargetTblName : STRING);
VAR         ExtCh     : CHAR;
            FldName   : STRING;
            NumFields : INTEGER;
            CaseSen   : BOOLEAN;
            FldArray  : FieldNumberArray;
            IndexId   : FieldNumber;
BEGIN
 ExtCh := '0';
 WHILE (ExtCh <= '9') AND
       (dDatabase.GetCompKeyInfo(StructTblName + '.xg' + ExtCh,
                                 FldName,
                                 NumFields,
                                 CaseSen,
                                 FldArray,
                                 IndexId) = PXSUCCESS) DO
  BEGIN
   WRITELN('Converting ' + StructTblName + '.xg' + ExtCh,'  ',
           FldName,NumFields:6,CaseSen:8,IndexId:6);
   PxErrMsg(dDatabase.DefineCompoundKey(TargetTblName,
                                        NumFields,
                                        FldArray,
                                        FldName,
                                        CaseSen,
                                        IndexId),'DefineCompoundKey');
   PxErrMsg(dDatabase.CreateSIndex(TargetTblName,
                                   IndexId,
                                   pxIncSecondary),'CreateSIndex');
   INC(ExtCh)
  END
END;

PROCEDURE   ConvertTable(SourceTblName,
                         TargetTblName,
                         LikeTblName   : STRING ;
                         BlockSize     : INTEGER);
VAR         PrimaryFields : INTEGER;
            TableDesc     : PTableDesc;
            StructTblName : STRING;
BEGIN
 IF ExistFile(TargetTblName + '.db') THEN
  PxErrMsgHalt(PXERR_CANTUPGRADE,'Target table already exists');
 InitPxEngine(BlockSize);
 TableDesc := dDatabase.GetDescVector(SourceTblName);
 IF TableDesc <> NIL THEN
  BEGIN
   PxErrMsgHalt(dDatabase.CreateTable(TargetTblName,TableDesc),'CreateTable');
   IF LikeTblName <> '' THEN StructTblName := LikeTblName
   ELSE StructTblName := SourceTblName;
   PrimaryFields := dDatabase.GetNumPFields(StructTblName);
   PxErrMsg(dDatabase.LastError,'GetNumPFields');
   IF PrimaryFields > 0 THEN
    BEGIN
     WRITELN('Creating primary index on ',PrimaryFields,' fields');
     PxErrMsg(dDatabase.CreatePIndex(TargetTblName,PrimaryFields),'CreatePIndex');
     ConvertSecondaryIndexes(StructTblName,TargetTblName);
     ConvertCompositeIndexes(StructTblName,TargetTblName)
    END;
   WRITELN('Copying records from ',SourceTblName, ' to ',TargetTblName);
   PxErrMsg(dDatabase.AppendTable(SourceTblName,TargetTblName),'AppendTable');
   DISPOSE(TableDesc,Done)
  END
 ELSE PxErrMsgHalt(dDatabase.LastError,'GetDescVector');
 DonePxEngine;
END;

PROCEDURE   Error;
BEGIN
 WRITELN;
 WRITELN('Usage:      BLCKSIZE SourceTable TargetTable NewBlockSize [LikeTable]');
 WRITELN;
 WRITELN('Where:      SourceTable is the Paradox table to be converted');
 WRITELN('            TargetTable is the name of the converted table');
 WRITELN('            NewBlockSize is the desired block size (1, 2, 3, or 4)');
 WRITELN('            LikeTable provides the index structure used on TargetTable');
 WRITELN;
 WRITELN('Examples:   BLCKSIZE customer newcust 1');
 WRITELN('            BLCKSIZE customer c:\newtbls\customer 4');
 WRITELN;
 WRITELN('Info:       Block sizes of 1, 2, 3, and 4 correspond to maximum');
 WRITELN('            table sizes of 64mb, 128mb, 192mb, and 256mb, respectively');
 WRITELN;
 WRITELN('Disclaimer: Use at your own risk');
END;

PROCEDURE   Process;
VAR         S         : STRING;
            BlockSize : INTEGER;
BEGIN
 WRITELN;
 WRITELN('BLCKSIZE 1.01:  Block Size Converter For Paradox Tables');
 WRITELN('Copyright (c) 1994 Tamarack Associates.  All rights reserved.');
 IF (ParamCount < 3) OR (ParamCount > 4) THEN Error
 ELSE
  IF Str2Int(ParamStr(3),BlockSize) THEN
   {* Strip off any extension *}
   IF ParamCount = 3 THEN
    ConvertTable(RemoveExt(ParamStr(1)),
                 RemoveExt(ParamStr(2)),'',
                 BlockSize)
   ELSE
    ConvertTable(RemoveExt(ParamStr(1)),
                 RemoveExt(ParamStr(2)),
                 RemoveExt(ParamStr(4)),
                 BlockSize)
  ELSE Error;
 WRITELN
END;

BEGIN
 Process
END.
